﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

namespace QueueDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            
            /* Note, to run these samples, you need an Azure Service Bus account,
             * an active Azure Service Bus namespace and an ACS V2 namespace.
             * Please go to http://Azure.com for more information on getting started.
             */ 

            string issuer = "owner";
            string key = "Your Key";
            string serviceNamespace = "Your Namespace";
            const string acsHostName = "Your ACS Namespace";
            string queueName = "Orders";


            /*********************************
             *** Administrative Operations ***
             *********************************/

            // Configure and create NamespaceManager for performing administrative operations
            NamespaceManagerSettings settings = new NamespaceManagerSettings();
            TokenProvider tokenProvider = settings.TokenProvider = TokenProvider.CreateSharedSecretTokenProvider(issuer,key);
            
            NamespaceManager manager = new NamespaceManager(ServiceBusEnvironment.CreateServiceUri("sb", serviceNamespace, string.Empty), settings);

            // Check for existence of queues on the fabric
            if(!manager.QueueExists(queueName))
            {
                Console.WriteLine("Queue does not exist");
                
                // Create Queue
                Console.WriteLine("Creating Queue...");
                manager.CreateQueue(new QueueDescription(queueName) { LockDuration = TimeSpan.FromSeconds(5.0d) });
            }
           

            /**************************
             *** Runtime Operations ***
             **************************/


            // Create and Configure Messaging Factory to provision QueueClient
            MessagingFactorySettings messagingFactorySettings = new MessagingFactorySettings();
            messagingFactorySettings.TokenProvider = settings.TokenProvider = TokenProvider.CreateSharedSecretTokenProvider(issuer, key);
            MessagingFactory messagingFactory = MessagingFactory.Create(ServiceBusEnvironment.CreateServiceUri("sb", serviceNamespace, string.Empty), messagingFactorySettings);
            
            QueueClient queueClient = messagingFactory.CreateQueueClient(queueName, ReceiveMode.PeekLock);

            Order order = new Order();
            order.OrderId = 42;
            order.Products.Add("Kinect", 70.50M);
            order.Products.Add("XBOX 360", 199.99M);
            order.Total = order.Products["Kinect"] + order.Products["XBOX 360"];

            // Create a Brokered Message from the Order object
            BrokeredMessage msg = new BrokeredMessage(order);

            /***********************
            *** Send Operations  ***
            ************************/

            queueClient.Send(msg);


            /* Alternate Message Sender */

            // Option 1
            //MessageSender sender = messagingFactory.CreateMessageSender(queueName);
            //BrokeredMessage msg2 = new BrokeredMessage(order);
            //sender.Send(msg2);

            //// Option 2
            //MessageSender sender2 = queueClient.MessagingFactory.CreateMessageSender(queueName);
            //BrokeredMessage msg3 = new BrokeredMessage(order);
            //sender.Send(msg3);




            /**************************
             *** Receive Operations ***
            ***************************/
            
            BrokeredMessage recdMsg;
            Order recdOrder;

            // Receive and lock message
            recdMsg = queueClient.Receive();

            if(recdMsg != null)
            {
                // Convert from BrokeredMessage to native Order
                recdOrder = recdMsg.GetBody<Order>();

                Console.ForegroundColor = ConsoleColor.Green;
                Console.WriteLine("Received Order {0} \n\t with Message Id {1} \n\t and Lock Token:{2} \n\t from {3} \n\t with total of ${4}", recdOrder.OrderId, recdMsg.MessageId, recdMsg.LockToken, "Receiver 1", recdOrder.Total);

                // Simulate something going wrong after peeking message
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("Crash! Exception!");

                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("Attempting to receive again at " + DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second);
            }

            // Alternate receive approach using agnostic MessageReceiver
            MessageReceiver receiver = messagingFactory.CreateMessageReceiver(queueName);

            // Recieve again, complete, and delete message from the fabric
            try
            {
                // Receive and lock message
                recdMsg = receiver.Receive();

                // Convert from BrokeredMessage to native Order
                recdOrder = recdMsg.GetBody<Order>();

                // Complete read, release and delete message from the fabric
                receiver.Complete(recdMsg.LockToken);

                Console.ForegroundColor = ConsoleColor.Green;
                Console.WriteLine("Received Order {0} \n\t with Message Id {1} \n\t and Lock Token:{2} \n\t from {3} \n\t with total of ${4} \n\t at {5}", recdOrder.OrderId, recdMsg.MessageId, recdMsg.LockToken, "Receiver 2", recdOrder.Total, DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second);
            }
            catch
            {
                // Should processing fail, release the lock from the fabric and make message available for later processing.
                if (recdMsg != null)
                {
                    receiver.Abandon(recdMsg.LockToken);
                    
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine("Message could not be processed.");

                }
            }
            finally
            {
                receiver.Close();
            }

            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine("Hit any key to exit.");
            Console.ReadLine();
        }
    }
}
